@@ -20,7 +20,7 @@ from .cpp import RENAME_METHOD_SUFFIX, RENAME_FIELD_SUFFIX
 from .cpp import cppname, cppnames, absname, typename, findClass
 from .cpp import line, signature, find_method
 from .cpp import Modifier, Class, Method
-from .cpp import getActualTypeArguments, getTypeParameters
+from .cpp import getActualTypeArguments, getTypeParameters, getRang
 from .config import INCLUDES, CFLAGS, DEBUG_CFLAGS, LFLAGS, IMPLIB_LFLAGS, \
     SHARED, VERSION as JCC_VER
 
@@ -734,7 +734,7 @@ def python(env, out_h, out, cls, superClasses, names, superNames,
 
     for name, methods in allMethods:
         args, x, cardinality = methodargs(methods, superMethods)
-        methods.sort(key=lambda x: len(x.getParameterTypes()))
+        methods.sort(key=lambda x: (len(x.getParameterTypes()), getRang(x.getParameterTypes()))) # Sort on both parameter length and order in class tree
         method = methods[0]
         modifiers = method.getModifiers()
         if name == 'iterator' and iteratorMethod is None:
@@ -1626,7 +1626,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         line(out, 0, "import os")
         line(out, 0, "from . import %s", extname)
     line(out)
-    line(out, 0, '__dir__ = os.path.abspath(os.path.dirname(__file__))')
+    line(out, 0, '__module_dir__ = os.path.abspath(os.path.dirname(__file__))')
 
     package_data = []
     for jar in jars:
@@ -1705,7 +1705,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         line(out)
         line(out, 0, 'VERSION = "%s"', version)
 
-    line(out, 0, 'CLASSPATH = [%s]' %(', '.join(['os.path.join(__dir__, "%s")' %(os.path.basename(jar)) for jar in jars])))
+    line(out, 0, 'CLASSPATH = [%s]' %(', '.join(['os.path.join(__module_dir__, "%s")' %(os.path.basename(jar)) for jar in jars])))
     line(out, 0, 'CLASSPATH = os.pathsep.join(CLASSPATH)')
     line(out, 0, '%s.CLASSPATH = CLASSPATH', extname)
     line(out, 0, '%s._set_function_self(%s.initVM, %s)',
@@ -1722,7 +1722,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
     includes = [os.path.join(output, extname),
                 os.path.join(jccPath, 'sources')]
     for import_ in imports:
-        includes.append(os.path.join(import_.__dir__, 'include'))
+        includes.append(os.path.join(import_.__module_dir__, 'include'))
 
     sources = ['JObject.cpp', 'JArray.cpp', 'functions.cpp', 'types.cpp']
     if not shared:
@@ -1843,7 +1843,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
                 ' '.join(IMPLIB_LFLAGS) %(os.path.join(modulePath, extlib))
             ]
             args['libraries'] = [
-                os.path.join(import_.__dir__, 'lib',
+                os.path.join(import_.__module_dir__, 'lib',
                              '_%s%s' %(import_.__name__, _d))
                 for import_ in imports
             ]
